/**
 * This file contains all the callbacks triggered by the notificationListener.
 * Also general callbacks are defined which are doing some hiding/disabling and
 * showing/enabling of common elements when data is being transmitted or not.
 */

// Callback to disable some elements for the user when the client is busy.
function disableElementsGeneralCallback(resultsList, detail) {
  if (detail.client.isBusy) {
    resultsList.fullResultsCreate.classList.toggle('disabled', true);
    resultsList.subResultsCreate.classList.toggle('disabled', true);
    resultsList.toggleInspectButtons(detail.client);
  }
}

// Callback to enable some elements for the user when the client is not busy.
function enableElementsGeneralCallback(resultsList, detail) {
  if (!detail.client.isBusy) {
    resultsList.fullResultsCreate.classList.toggle('disabled', false);
    if (Object.keys(resultsList.subResultsIndexes).length > 0) {
      resultsList.subResultsCreate.classList.toggle('disabled', false);
    }
    resultsList.toggleInspectButtons(detail.client);
  }
}

 /**
 * Callback opening the loading modal when the client is connecting to the
 * CQP server.
 */
function connectingCallback(resultsList, detail) {
  resultsList.getHTMLElements(['#analysis-init-modal']);
  resultsList.analysisInitModal = M.Modal.init(resultsList.analysisInitModal,
                                               {dismissible: false});
  resultsList.analysisInitModal.open();
}

// Callback that closes the loading modal from above.
function connectedCallback(resultsList, detail) {
  /**
   * In the past this closed the init modal. But the init modal is now being
   * closed when the meta data has also been recieved. See below.
   */
}

// Callback that closes the loading modal from above.
function metaDataRecievedCallback(resultsList, detail) {
  resultsList.analysisInitModal.close();
}

// Callback that shows the user some feedback if the client raised an error.
function clientFailedCallback(resultsList, detail) {
  resultsList.getHTMLElements([
    '#analysis-init-progress',
    '#analysis-init-error',
    '#user-feedback',
  ]);
  if (detail.raiseModalFeedback) {
    resultsList.analysisInitModal.open();
    resultsList.analysisInitProgress.classList.toggle('hide');
    resultsList.analysisInitError.classList.toggle('hide');
    resultsList.analysisInitError.textContent = detail.msg;
  } else {
    nopaque.appClient.flash(detail.msg, 'error')
  }
}

// Callback doing some preperation work if a query has been issued by the user.
function queryDataPreparingCallback(resultsList, detail) {
  // remove all items from resultsList, like from the query issued before
  resultsList.clear()
  // get needed HTML Elements
  let results = detail.results;
  resultsList.getHTMLElements([
    '#interactions-menu',
    '#recieved-match-count',
    '#total-match-count',
    '#text-lookup-titles',
    '#text-lookup-count',
    '#query-results-user-feedback',
    '#query-progress-bar',
    '#query-results-create',
    '#sub-results-indexes-display',
    '#nr-marked-matches',
  ]);
  // show or enable some things for the user
  resultsList.interactionsMenu.classList.toggle('hide', false);
  resultsList.queryResultsUserFeedback.classList.toggle('hide', false);
  resultsList.queryProgressBar.classList.toggle('hide', false);
  resultsList.showCorpusFiles.classList.toggle('disabled', true);
  /**
   * Set some initial values for the user feedback
   * or reset values for new issued query
   */
  resultsList.recievedMatchCount.textContent = 0;
  resultsList.totalMatchCount.textContent = results.data.match_count;
  resultsList.textLookupCount.textContent = 0;
  resultsList.nrMarkedMatches.textContent = 0;
  resultsList.subResultsIndexesDisplay.textContent = '';
  resultsList.resetFields();
}

/**
 * Callback handling the incoming results of an issued query. It renders
 * the incoming matches using the resultsList for the user.
 */
function queryDataRecievingCallback(resultsList, detail) {
  // load the data into the resultsList and show them to the user
  let results = detail.results;
  let client = detail.client;
  let start = detail.dataLength;
  let resultItems = [];
  for (let [index, match] of Object.entries(results.data.matches).slice(start)) {
    resultItems.push({ ...match, ...{ 'index': parseInt(index) } });
  }
  if (client.dynamicMode) {
    resultsList.add(resultItems, (items) => {
      for (let item of items) {
        item.elm = resultsList.createResultRowElement(item,
                                                      results.data,
                                                      client);
      }
    });
    // update user feedback about query status
    resultsList.recievedMatchCount.textContent = results.data.matches.length;
    resultsList.queryProgressBar.firstElementChild.style.width = `${client.requestQueryProgress}%`;
    resultsList.textLookupCount.textContent = `${Object.keys(results.data.text_lookup).length}`;
    // updating table on finished item creation callback via createResultRowElement
    resultsList.update();
    resultsList.changeHitsPerPage(client, results);
    resultsList.changeContext();
    //activate expertMode of switch is checked
    if (resultsList.displayOptionsFormExpertMode.checked) {
      resultsList.expertModeOn('query-display', results);
    }
  } else if (!client.dynamicMode) {
    resultsList.add(resultItems, (items) => {
      for (let item of items) {
        item.elm = resultsList.createResultRowElement(item,
                                                      results.data,
                                                      client,
                                                      true);
      }
    });
    // update user feedback about query status
    resultsList.recievedMatchCount.textContent = results.data.matches.length;
    resultsList.queryProgressBar.firstElementChild.style.width = `${client.requestQueryProgress}%`;
    resultsList.textLookupCount.textContent = `${Object.keys(results.data.text_lookup).length}`;
    // updating table on finished item creation callback via createResultRowElement
    resultsList.update();
    resultsList.changeHitsPerPage(client, results);
    resultsList.changeContext();
  }
}

// Callback that is executed when all results from an issued query have been recieved
function queryDataRecievedCallback(resultsList, detail) {
  // hide or disable some things for the user
  resultsList.queryResultsUserFeedback.classList.toggle('hide', true);
  resultsList.queryProgressBar.classList.toggle('hide', true);
  // reset bar progress for next query
  resultsList.queryProgressBar.firstElementChild.style.width = '0%';
  resultsList.showCorpusFiles.classList.toggle('disabled', false);
}

/**
 * Callback that is handling incoming results data. Results data is needed for
 * the export and download of the data.
 */
function resultsDataRecievingCallback(resultsList, detail) {
  resultsList.getHTMLElements([
    '#full-results-progress-bar',
    '#sub-results-progress-bar',
  ]);
  // Disable the full context switch when results are being recieved.
  resultsList.exportFullInspectContext.setAttribute('disabled', '');
  if (detail.type === 'full-results' && detail.progress) {
    resultsList.fullResultsProgressBar.firstElementChild.style.width = `${detail.progress}%`;
    resultsList.fullResultsProgressBar.classList.toggle('hide', false);
  } else if (detail.type === 'sub-results' && detail.progress) {
    resultsList.subResultsProgressBar.firstElementChild.style.width = `${detail.progress}%`;
    resultsList.subResultsProgressBar.classList.toggle('hide', false);
  }
}
/**
 * Callback is executed when all results data has been recieved.
 * Reactivates the resutls create buttons etc.
 */
function resultsDataRecievedCallback(resultsList, detail) {
  // create strings for create buttons depending on type
  const handleType = (keyPrefix, text) => {
    // Enable the full context switch when results have been recieved
    resultsList.exportFullInspectContext.removeAttribute('disabled', '');
    // hides the create element after results have been recieved and reset it
    resultsList[`${keyPrefix}Create`].classList.toggle('hide');
    resultsList[`${keyPrefix}Create`].textContent = `Create ${text}`;
    resultsList[`${keyPrefix}Create`].insertAdjacentHTML('beforeend',
    `<i class="material-icons left">build</i>`);
    // show and highlight export button
    resultsList[`${keyPrefix}Export`].classList.toggle('hide', false);
    resultsList[`${keyPrefix}Export`].classList.toggle('pulse', true);
    setTimeout(() => {
      resultsList[`${keyPrefix}Export`].classList.toggle('pulse', false);
      clearTimeout();
    }, 3000)
  }
  if (detail.type === 'full-results') {
    handleType('fullResults', 'Results');
    if (detail.fullContext) {
      resultsList.fullResultsProgressBar.firstElementChild.style.width = `0%`;
      resultsList.fullResultsProgressBar.classList.toggle('hide', true);
    }
  } else if (detail.type ==='sub-results') {
    handleType('subResults', 'Sub-Results');
    if (detail.fullContext) {
      resultsList.subResultsProgressBar.firstElementChild.style.width = `0%`;
      resultsList.subResultsProgressBar.classList.toggle('hide', true);
    }
  } else if (detail.type ==='inspect-results') {
    if (Object.keys(resultsList.subResultsIndexes).length === 0) {
      /**
      * Prevent create sub results button from being activated if it is disabled
      * and no matches have been marked by the user for sub results creation.
      */
      resultsList.subResultsCreate.classList.toggle('disabled', true);
    }
    resultsList.showMatchContext(detail.results, detail.client);
  }
}

// export the callbacks
export {
  connectingCallback,
  connectedCallback,
  metaDataRecievedCallback,
  clientFailedCallback,
  queryDataPreparingCallback,
  queryDataRecievingCallback,
  queryDataRecievedCallback,
  resultsDataRecievingCallback,
  resultsDataRecievedCallback,
  disableElementsGeneralCallback,
  enableElementsGeneralCallback,
};
